const unsigned char Gen_[] = {0xFA,0x82,0xB9,0xAB,0xC3,0x6B,0x7B,0xA2,0xFB,0xEB};
                               // define digits

#define _U  0xDA           //----- define signes ------------------------
#define _u  0x1A
#define _P  0xF1
#define _Bottom 0x08
#define _Middle 0x01
#define _Space  0
#define _S  0x6B
#define _O  0xFA
#define _o  0x1B
#define _C  0x78           // segments of Sign to indicator - (b f a e d h c g )
#define _A  0xF3           //                           bits  (7 6 5 4 3 2 1 0 )
#define _L  0x58
#define _I  0x82
#define _F  0x71
#define _t  0x59
#define _n  0x13
#define _E  0x79
#define _r  0x11
#define _H  0xD3




unsigned char  Flags=0; /* bit 7 ... 0-common anode, 1-cathode
                           bit 0 ... 1 -some button is on (s1 or s2)
                           bit 1 ... 1 -s1 is on
                           bit 2 ... 1 -s2 is on
                           bit 3 ... used to auto-push button (to delay )
                           bit 4 ... 1 - the both buttons are pushed
                           bit 5 ... 0 - it was measure in carrent 0.5s
                      */
unsigned char  Sign[4] = {0,0,0,0}; // signs to OUT
unsigned char Regime, FlashFlags=0;
unsigned int volatile TimeCount=0;
unsigned short volatile Timer=0;



#define set(x,y)  x = x | (1<<y)         // set bit
#define point(x)  set(Sign[x],2)         // point to Sign x (set bit 2)
#define clear(x,y)  x = x &~(1<<y)       // clear bit
#define int_on INTCON = INTCON |(1<<GIE) // int. enable
#define int_off INTCON = INTCON &~(1<<GIE) // int. disable
#define s1_on  (PORTA.RA4==0)
#define s2_on  (PORTA.RA5==0)



 // Software I2C connections
sbit Soft_I2C_Scl           at RC3_bit;
sbit Soft_I2C_Sda           at RC4_bit;
sbit Soft_I2C_Scl_Direction at TRISC3_bit;
sbit Soft_I2C_Sda_Direction at TRISC4_bit;
// End Software I2C connections






//********************* Procedure of pause 4 - 1000 ms (by 4ms step) ***********
// In parameter - Delay (ms)
// Needs global variable Timer which increments in 4 ms interrups

void Pause_ms(unsigned char Delay ) {
  Delay = Delay/4; // intervals 4 ms
  Delay += Timer;
  while (Timer != Delay); // wait (Delay)*4 ms or (in-parameter) ms
 }
 
//******************************************************************************







/*
*******************************************************************************
        Procedure of read from EEPROM 24c02
=============================================================================
*/
unsigned char Eeprom_Read_(unsigned char x) {
  unsigned char Read_value;
  int_off;
  Soft_I2C_Start();               // Issue start signal
  Soft_I2C_Write(0xA0);           // Address 24c02 +write
  Soft_I2C_Write(x);              // address
  Soft_I2C_Start();               // Issue repeated start signal
  Soft_I2C_Write(0xA1);           // Address 24c02 +Read
  Read_value = Soft_I2C_Read(0);  // Read byte  with ask
  Soft_I2C_Stop();                // Issue stop signal
  int_on ;
  Pause_ms(50);

  return Read_value;

 }

//-----------------------------------------------------------------------------




//***************** Indication 'Error' *****************************************
//
void EE_Error(void) {
 TimeCount = 0;
 Sign[3]= _E;
 Sign[2]= _E;
 Sign[1]= _r;
 Sign[0]= _r;           //
 point(3);        // if EE-problems 'E.Err' to 2 sec.
 while (((unsigned char)TimeCount).F1==0);  // loop 2 sec.
 Sign[3]=_Space;
 Sign[2]=_Space;
 Sign[1]=_Space;
 Sign[0]=_Space;          // put out the indicator

}

//******************************************************************************




/*
=============================================================================
          Procedure of write to EEPROM 24c02 (adress, data)
=============================================================================
*/
void Eeprom_Write_(unsigned char x, unsigned char y) {
   unsigned char y_read;
   
   int_on;
   Pause_ms(50);
   int_off;
   Soft_I2C_Stop();                // Issue stop signal
   Soft_I2C_Start();               // Issue start signal
   Soft_I2C_Write(0xA0);           // Address 24c02 and write
   Soft_I2C_Write(x);              // Address to write
   Soft_I2C_Write(y);              // data
   Soft_I2C_Stop();                // Issue stop signal
   int_on ;
   Pause_ms(50);
                            // ---> test the writing
   int_off;
   Soft_I2C_Start();               // Issue start signal
   Soft_I2C_Write(0xA0);           // Address 24c02 +write
   Soft_I2C_Write(x);              // address
   Soft_I2C_Start();               // Issue repeated start signal
   Soft_I2C_Write(0xA1);           // Address 24c02 +Read
   y_read = Soft_I2C_Read(0);      // Read byte  with ask
   Soft_I2C_Stop();                // Issue stop signal
   int_on ;

   
   if (y != y_read) EE_Error();  //  --> mistake
   Pause_ms(50);

 }
//===========================================================================












//******************** Prototype Init_EE() ************************************
 unsigned char Init_EE(struct Sdata *s);
 
//*****************************************************************************





//***************** INT (<999) --> BCD to ind. )***(2400 cycles)****************

void Int_to_ind(unsigned int x) {
 unsigned char i, y, Digit[] = {0,0,0,0};

 if (x>999) return;
 Digit[2] = (char)(x/100);
 y = (char)(x%100);
 Digit[1] = y/10;
 Digit[0] = y%10;
 int_off;
 for (i=0; i<3; i++) Sign[i] = Gen_[Digit[i]]; // to ind.
 if (Sign[2]==_O) Sign[2] = _Space;          // put out the non signify null
 int_on;
 return;
 }

//******************************************************************************




 //************************* PWM off *******************************************
 
 void PWM1_10_off(void) {
  CCP1CON = CCP1CON&0xF0;
  clear(T2CON,TMR2ON);   // stop TMR2
  set(PORTC,RC2);        // switch off out
  set(TRISC,RC2);
  clear(TRISC,RC2);
  }
 //*****************************************************************************
 



  //************************* PWM on *******************************************

 void PWM1_10_on(void) {
  set(T2CON,TMR2ON);   //start TMR2
  CCP1CON = CCP1CON|0x0C;
  clear(TRISC,RC2); // RC2 - out
  }
 //*****************************************************************************

 
 



//********************* PWM-10 set Duty  ( in - 0-1023) ************************

void PWM1_10_Duty(unsigned int p_Duty) {
 CCPR1L = p_Duty>>2;                          // high 8 bit to CCPR1L
 CCP1CON = (((unsigned char)p_Duty)<<4)|(CCP1CON&0x0f); // low 2 bit to 5,6 bit
                                                        // of CCP1CON
 }

//******************************************************************************




//********* 10-bit PWM init.  (Fmax = Fosc/1000) prescaler=1, ******************
//****** Fmid = Fosc/4000 prescaler=4, Fmin = Fosc/16000) prescaler=16 *********

void PWM1_10_Init(unsigned char Pres_TMR2)  {
 unsigned char ToTMR2;
 switch (Pres_TMR2) {
   case 16:  ToTMR2 = 0x06; break;
   case 4:   ToTMR2 = 0x05; break;
   case 1:   ToTMR2 = 0x04; break;
   default:  ToTMR2 = 0x04;     // default prescaler value 1
   }

 // initialization PWM
 PR2 = 255;
 CCPR1L = 0;
 CCP1CON =0;
 T2CON = ToTMR2;   //  set prescaler
 }

//******************************************************************************




//************************* waiting not pushed in 32 ms buttons ****************

void Not_Pushed() {
  while (s1_on || s2_on);                // wait not pushed
  Timer = 0;
  while ((Timer.F3==0)||s1_on || s2_on); // wait not pushed in 32 ms
  }
//******************************************************************************









// **************** I or U out of range ****************************************
// In: 1 - I out of range, 0 - U

void Over_limit(char Par ) {

 int_on;
 if (Par==0) {
   Sign[3]= _U;                              //  'U_In'
   Sign[2]= _Bottom;                         //
   Sign[1]= _I;                              //
   Sign[0]= _n;                             //
   }
  else {
   Sign[3]= _I;                              //
   Sign[2]= _Bottom;                         //  'I_HI'
   Sign[1]= _H;                              //
   Sign[0]= _I;                              //
   }
   
 FlashFlags = 0x0F;               // all signs is flash


 while (1);                  // endless loop

 }

// *****************************************************************************














//****************** UI measure  (out - I, to (*(in)) - U ************************

unsigned int UI_meter(unsigned char *x_u) {
unsigned char x_i;
int_off;
PWM1_10_off();

(*x_u) = Adc_Read(0);     // VOLTAGE

if ((*x_u)==255) INTCON |= 0b00000011;       // U out of range --> interrupt (U)

clear(PORTC,RC2);          // RC2=0
TRISC=0xFF;
TRISC=0;
Delay_us(20);
x_i = Adc_Read(1);         // current
if (x_i==255) INTCON |= 0b00000001;          // I out of range --> interrupt (I)

set(PORTC,RC2);            // RC2=1
int_on;
if (x_i >= 160) (*x_u)--;         //  correction for dropout on R and MOSFET
return x_i;
}

//******************************************************************************







//************************** BUTTONS *******************************************
// In:  Byte to change, low limit, high limit
// Out: Changed or not changed byte
// Use increaseble in interrups var. TimeCount
// If both buttons pushed - bit 4 of Flags is set
//******************************************************************************

unsigned char Button_push(char OutVal, char LimLow, char LimHigh) {
char Timer1, Timer2;

Timer1 = TimeCount;    // 2-nd byte of 4ms counter
Timer2 = Timer;       // small 8 bit

if (Flags.F0==1) {
  if ((Flags.F1==0)&&(Flags.F2==0)) {         //
     if (Timer2<5)   return OutVal;// for false switching protect
     if s1_on set(Flags,1);   // s1 flag
     if s2_on set(Flags,2);   // s2 flag
     if (!s1_on && !s2_on) clear(Flags,0);//clear flag if buttons are not pushed
     }
     else {
        if (s1_on && s2_on) {Flags &= 0xF0; set(Flags,4);} // to menu --->
        if (Flags.F1==1) {
           if (!s1_on) {
              if (OutVal<=LimLow) {Flags &= 0xF0;return OutVal;}// if is limit
              OutVal--;               //
              Flags &= 0xF0;         // decrement a value
              }
              else {
                 if (Timer1>2) {    // time 3s to auto
                    if (Timer2.F6==0) {set(Flags,3); return OutVal;}
                    if (OutVal<=LimLow) return OutVal; // if is limit --> exit
                    if (Flags.F3==1) OutVal--; // decrease
                    clear(Flags,3);           // 1 time each 0.5s
                    }
                  }
            }
          }
        if (Flags.F2==1) {
           if (!s2_on) {
              if (OutVal >= LimHigh) {Flags &= 0xF0;return OutVal;} // if is limit
              OutVal++;               //
              Flags &= 0xF0;         // increment a value
              }
              else {
                 if (Timer1>2) {        //time 3s to auto
                    if (Timer2.F6==0) {set(Flags,3); return OutVal;}
                    if (OutVal>=LimHigh) return OutVal; // if is limit --> exit
                    if (Flags.F3==1) OutVal++; // increase
                    clear(Flags,3);           // 1 time each 0.5s
                    }
                  }
            }


 }
if ((Flags.F0==0)&&(s1_on||s2_on)) {TimeCount = 0; set(Flags,0);}
                        // clear timer and set flag if any button pushed
if ((Flags.F0==0) && !(s1_on||s2_on)) TimeCount=0;
                        // clear Timer after the push
return OutVal;

}


//******************************************************************************





//******************** switch off the output if time is out ********************

 char Time_To_Off (int Time) {
  Time *= 360;
  if (Time < TimeCount) {
     PWM1_10_off();         // turn off out
     Sign[0] = _F;   //
     Sign[1] = _F;   //
     Sign[2] = _o;   // oFF
     while (!s1_on && !s2_on); // wait to pushing
     TimeCount = 0;            // reset timer
     PWM1_10_on();       // on
     Not_Pushed();                   // wait to not pushing
     return 0;
     }
     
  return 1;                // no was sleep
  
  }


//*************************Prototype of Menu ***********************************
// Change the parameters FStartU, FStartTime, TimeOff
// Starting the calibration menu


 void Menu (struct Sdata *s);

 //-----------------------------------------------------------------------------



 
 
 








//********* Prototipe of Menu  change the Regime and set parameters ************
// in - point of stucture Sdata
// push the both buttons - next menu

void Menu_regime (struct Sdata *s);
//******************************************************************************






//********************** Calibration procedure *********************************
// in - pointer of CalibrU and CalibrI

void Calibration (unsigned char *CalibrU, unsigned char *CalibrI) {
  unsigned int  Out_x;
  unsigned char U_adc, I_adc;

  flashFlags=0;
  Not_Pushed() ;    // not pushed buttons waiting
  TimeCount = 0;
  Sign[3]=_t;           //
  Sign[2]=_o;           //
  Sign[1]=_u;           //
  Sign[0]=_t;           //
  point(3);       //  't.out' - turn out
  FlashFlags = 0x0F;    // all digits is flash

  while (1) {
    I_adc = UI_meter(&U_adc);
    if (I_adc < 3) break;  //go farther if output disconnected
    if (((short)TimeCount).F4 != 0) {FlashFlags=0; return;} //exit after 16 sec.
    Pause_ms(100);
   }
                // The load is off. Begining the Calibration U
  PWM1_10_off();
  clear(PORTC,RC2); // open the output
  FlashFlags=0;
  
                                  // Calibration U -----------------------------
  Sign[3] = _u;
  TimeCount=0;
  while (1) {

    if ((Flags.F0==1)||s1_on||s2_on )  (*CalibrU)=Button_push((*CalibrU),90,180);
              // if pushed  button or  it in treat --> change with limits 90,180

    if (Adc_read(1)>2) return; // exit if the load is on
    
    U_adc = Adc_Read(0);     // VOLTAGE
    
    Out_x = (unsigned int)(((unsigned long)U_adc)*100*(*CalibrU)/8313);
                                                 // U=(100/8313)*Nu*Ku
  //  Out_x=((unsigned int)U_adc)*(*CalibrU)/83;
    Int_to_ind(Out_x);  // to indicator

    if (((((short)TimeCount).F4 != 0)&&(Flags.F0==0))||(Flags.F4==1)) break; 
                    //exit after 16 sec. without push or pushed both
   }

  Eeprom_Write_(2, (*CalibrU));  // save to EEprom
  Flags &= 0b11100000;  // clear button Flags
  set(PORTC,RC2); // close MOSFET    --------------------------------------------
  
  Not_Pushed() ;    // not pushed buttons waiting
  TimeCount = 0;
  Sign[3]=_t;           //
  Sign[2]=_Bottom;      //
  Sign[1]=_o;           //
  Sign[0]=_n;           //
                        //  't_on' - turn on
  FlashFlags = 0x0F;    // all digits is flash

  while (1) {
    I_adc = UI_meter(&U_adc);
    if (I_adc > 3) break;  //go farther if output is connected
    if (((short)TimeCount).F4 != 0) {FlashFlags=0; return;} //exit after 16 sec.
    Pause_ms(100);
   }

                 // The load is on. Begining the Calibration I
                 // Calibration I ----------------------------------------------
  TimeCount=0;
  while (((char)TimeCount).F0==0); // wait 1 sec.
  FlashFlags=0;
  while (1) {

    if ((Flags.F0==1)||s1_on||s2_on )  (*CalibrI)=Button_push((*CalibrI),90,180);
              // if pushed  button or  it in treat --> change with limits 90,180
    I_adc = UI_meter(&(U_adc));
    if (I_adc < 3) { Flags &= 0b11100000; return;}//exit if disconnect the load
    
    Out_x = (unsigned int)((((unsigned long)U_adc)*(*CalibrU)*520/I_adc)/(*CalibrI));
                                               // R=520*Nu*Ku/(Ni*Ki)
    Sign[3] = Gen_[(short)(Out_x/1000)];
    if (Sign[3]==_O) Sign[3] = _Space;   // switch out ind. if null
  
    Out_x = Out_x % 1000;
    Int_to_ind(Out_x);                // Indicated the solderer resistanse (Ohm)
    if (Sign[2]==_Space) Sign[2] = _O;// here null is signify

    if (((((short)TimeCount).F4 != 0)&&(Flags.F0==0))||(Flags.F4==1)) break;
                    //exit after 16 sec. without push or pushed both
    Pause_ms(100);                                          // or pushed both
    }

  Eeprom_Write_(3, (*CalibrI));  // save to EEprom
  Flags &= 0b11100000;  // clear button Flags

}//-----------------------------------------------------------------------------



//******************************************************************************








// ************************** THE MAIN PROGRAMM ********************************

void main() {
unsigned char i, Temp, U_adc=0, U_adc_temp=0, I_adc_temp=0, I_adc=0, Out_U1, Out_U1_Temp;
unsigned int  P_toInd, Pwm_x;
unsigned long x, y;
struct Sdata {unsigned char CalibrU, CalibrI, Out_U, FStartU, FStartTime, TimeOff;} Sets;

TRISA = 0xFF;  // all - in
TRISB = 0xFF;
TRISC = 0;  // all - out
PORTA = 0;
PORTB = 0;

     //--------- Determin. anode-cathode and set int. interval  ----------
Temp = Clock_MHz();
while (1) {                           //
 if (Temp<2) {OPTION_REG = 1; break;} //  RB pullup on.
 if (Temp<3) {OPTION_REG = 2; break;} //  prescaler 4, 8, 16, 32 or 64
 if (Temp<6) {OPTION_REG = 3; break;} //  for int. interval 4ms
 if (Temp<12) {OPTION_REG = 4; break;}//  depended from Clock (MHz).
 OPTION_REG = 5; break;                    //  (Best to use quarz
 }                                         //  to 1, 2, 4, 8 or 16 MHz)
 
PORTC = 7;    // begining of determining anode-cathode

ADCON1 = 0x02; // AN0-AN4 - analog inputs, Udd - vref
i = Adc_Read(2);
if (i<200) Flags = Flags|(1<<7); // common cathode - set bit7 of Flags
               //-------------------------------------------------------------
PORTC = 4; if (Flags.F7 != 0) PORTC = 0xFF;   // hi - cathode, low - anode

ADCON1 = 0x05; // AN0-AN1 - analog inputs, AN3 - vref

PORTB = 0;
TRISB = 0b10000000; //  RB7 - in, all another - out


INTCON = 0b10100000;   // int_TMR0 on  , begining indication

Soft_I2C_Init();       // init. I2C

 Temp = Init_EE(&Sets);   // const. from EEprom  (1-ok, 0-default)
 if (Temp==0) EE_Error(); // if EE-problems 'E.Err' to 8 sec.

 Pwm_x = 1022;
 PWM1_10_Init(16);  // init. pwm 1000Hz (prescaler =16)
 PWM1_10_Duty(Pwm_x);
 

 Temp = PORTB;            //
 clear(INTCON,RBIF);      //
 set(INTCON,RBIE);        // int_RB on

 
 I_adc = UI_meter(&U_adc);
 PWM1_10_on();

 if (Regime.F0==0) Temp = _P;          //
     else  Temp = _U;                 //
 Sign[3] = Temp;  point(3);    // Regime identificator


// ............................Soft start and forsed heat.......................

 Temp = Sets.FStartTime; 
 if(Sets.FStartTime<30) {Temp = 0; Sets.FStartU = Sets.Out_U;}

 for ( Out_U1=45; Out_U1 < Sets.FStartU; Out_U1 += 10) { // ---SOFT START------
 
    x = 7069940; y = ((unsigned long)U_adc)*(Sets.CalibrU);  //
    x/=y;                                                   // Calculation duty
 //   x = x * ((unsigned long)Out_U1)*Out_U1;               // of Pwm from Uout
    x = x * Out_U1*Out_U1;               // of Pwm from Uout
    Pwm_x = (unsigned int)(x/y);
    if (Pwm_x>1022) {Pwm_x=0; Out_U1--; break;}
                                                     // if max Pwm - exit
       else {Pwm_x = 1023-Pwm_x;}

    PWM1_10_Duty(Pwm_x);
    
    clear(Flags,5);                          //
    while ((Timer.F4!=1)||(Flags.F5!=1)){  //
      if (Timer.F4!=1) set(Flags,5);     //
    }                                  //  1 time in 128 ms

       
   }//--------------------------------------------------------------------------

  Out_U1 = Sets.FStartU;
  TimeCount = 0;
  FlashFlags = 0x0F;  // flash on

  while ( ((short)TimeCount)< Temp) { //--------- FORSE START ------------------

    if (s1_on || s2_on) break;   // exit if button pushed

    if ((Timer.F6==1)&&(Flags.F5==1)) { //
      I_adc = UI_meter(&U_adc);         //
      PWM1_10_on();                     //
      clear(Flags,5);                   //
      }                                 // measure I and U each 0.5s
    if (Timer.F6==0) set(Flags,5);      //

    if ((U_adc==U_adc_temp)&&(I_adc==I_adc_temp)&&(Out_U1_Temp==Out_U1)) continue;
                                               // missing below  if no changes
    U_adc_temp = U_adc; I_adc_temp = I_adc; Out_U1_Temp = Out_U1;

                                    // x = 1023-((7069940/(Nu*Ku))*U^2)/(Nu*Ku)
    x = 7069940; y = ((unsigned long)U_adc)*(Sets.CalibrU);//----------------
    x/=y;                                                   // Calculation duty
    x = x * ((unsigned long)Out_U1)*Out_U1;                // of Pwm from Uout
    Pwm_x = (unsigned)(x/y);
    if (Pwm_x>1022) {Pwm_x=0; Out_U1--;}
                                                 // if max Pwm - flash 3-th bit
      else Pwm_x = 1023-Pwm_x;
                              // Nu - adc value, Ku - correction coeff.(90-180)

    PWM1_10_Duty(Pwm_x);                             //----------------------


     x = ((unsigned long)U_adc)* Sets.CalibrU*(1023-Pwm_x)/1024; //
     x = x * Sets.CalibrI / 113;                               //
     P_toInd = (unsigned int)(x * I_adc / 3179);            //
     if (P_toInd>990)  Out_U1--;                           // calculation and
 
     if (regime.F0==0) {                               //
        Int_to_ind(P_toInd);                         // indication if
        point(1);                            //  Regime is 'Power'
       }
        else Int_to_ind(Out_U1);

   }
 //.............................................................................

 Not_Pushed();
 Out_U1 = Sets.Out_U;  //load out voltage
 Sets.Out_U --;        // enable indication
 FlashFlags = 0;

 while (1) {       // ======================== the base cycle ==================
 
  if ((Timer.F6==1)&&(Flags.F5==1)) { //
    I_adc = UI_meter(&U_adc);         //
    PWM1_10_on();                     //
    clear(Flags,5);                   //
    }                                 // measure I and U each 0.5s
  if (Timer.F6==0) set(Flags,5);      //
  
 
  if ((Flags.F0==1)||s1_on||s2_on )  Out_U1 = Button_push(Sets.Out_U, 50, 250);
                                       // if pushed  button or  it in treat
                                       
  if ((Sets.Out_U!=Out_U1)||(U_adc!=U_adc_temp)||(I_adc)!=I_adc_temp) {
                                       // Do if there is changes
     Sets.Out_U = Out_U1; U_adc_temp = U_adc; I_adc_temp = I_adc;

                                     // x = 1023-((7069940/(Nu*Ku))*U^2)/(Nu*Ku)
     x = 7069940; y = ((unsigned long)U_adc)*(Sets.CalibrU);//----------------
     x/=y;                                                   // Calculation duty
     x = x * ((unsigned long)Sets.Out_U)*Sets.Out_U;        // of Pwm from Uout
     Pwm_x = (unsigned)(x/y);
     if (Pwm_x>1022) {Pwm_x=0; set(FlashFlags,3); Out_U1--;}
                                                 // if max Pwm - flash 3-th bit
        else {Pwm_x = 1023-Pwm_x; clear(FlashFlags,3);}
                              // Nu - adc value, Ku - correction coeff.(90-180)
                               
     PWM1_10_Duty(Pwm_x);                             //----------------------
    
    
     x = ((unsigned long)U_adc)* Sets.CalibrU*(1023-Pwm_x)/1024; //
     x = x * Sets.CalibrI / 113;                               //
     P_toInd = (unsigned int)(x * I_adc / 3179);            // calculation P
     if (P_toInd>990)  Out_U1--;                         //
      
     if (regime.F0==0) {                              //
        Int_to_ind(P_toInd);                        // indication if
        point(1);                                 //  Regime is 'Power'
       }
        else Int_to_ind(Sets.Out_U);            // is 'Voltage'


   }


  if (TimeCount==600) {                           // save Out_U in 10 min
        Eeprom_Write_(4,Sets.Out_U);            // after pushing any button.
        TimeCount++;                          // write 1 time
       }
       
  if (Flags.F4==1) {                // pushed both buttons
      Temp = Regime;
      Menu_regime(&Sets);        //
      Sets.Out_U--;             //  menu and activate indication
      if (Temp!=Regime) Eeprom_Write_(1,Regime); // if Regime is changed
                                                 // save current regime
      if (Regime.F0==0) Temp = _P;
          else  Temp = _U;
      Sign[3] = Temp;  point(3);    // Regime identificator
     }

  if (Time_To_Off(Sets.TimeOff)==0) Sets.Out_U--; //out voltage to off if time out
                                                // if was sleep activate ind.
                                                
 } //========================= end of base cycle ===============================
}



//************************* END OF MAIN PROGRAMM *******************************







//**************************** INTERRUPTS **************************************
void interrupt(void) {

  unsigned char To_portc=0, To_portb=0, Temp_portc;
  static unsigned char Col_ = 1;         // column of indicator

  if (INTCON.TMR0IF = 1)
     {
      Timer +=1 ;                          // 4ms counter
      if (Timer==0) TimeCount +=1;        // 1 s counter  (16 bit)

      if (Col_.F3 != 0) Col_ = 1;  else Col_ <<= 1;  // 0001-0010-0100-1000-0001

      To_portc = PORTC;
      To_portc &= 0b00011101;                  // (000xxx0x)
      Temp_portc = To_portc ^ 0b11100010;      // (111xxx1x)
      if (Flags.F7 != 0) To_portc = Temp_portc;
                                     //  anode -(000xxx0x)/ cathode -(111xxx1x)
      PORTC = To_portc;  // ind. off

      if (Timer.F7==1) To_portc = Col_ & ~FlashFlags; //
         else To_portc = Col_;                                   // doing flash

      To_portc = ((To_portc<<4)|(To_portc<<1))|0b00011101; // 0000xxxx --> xxx111x1
      To_portc &= Temp_portc;
      if (Flags.F7 != 0) To_portc ^= 0b11100010;   // if anode (~)

      if (Col_.F0 == 1) To_portb = Sign[0];
      if (Col_.F1 == 1) To_portb = Sign[1];
      if (Col_.F2 == 1) To_portb = Sign[2];
      if (Col_.F3 == 1) To_portb = Sign[3];
      if (Flags.F7 == 0) To_portb =~ To_portb;   // if anode
      PORTB = To_portb | 0b10000000;       // 1xxxxxxx
      
      clear(To_portc,0);                  //
      if (To_portb.F7==1) set(To_portc,0);// RB7 --> RC0

      PORTC = To_portc;  // ind. --> on

      clear(INTCON,TMR0IF);      // clear flag TMR0IF
      }



  if ((INTCON.RBIF==1)&&(INTCON.RBIE==1)) {        // Int. by carrent overlimit
   
     clear(INTCON,RBIE);                           // only 1 calling
     PWM1_10_off();
     
     To_portb = PORTB;
     clear(INTCON,RBIF);

     if (INTCON.INTF==1) Over_limit(0);             // voltage overlimit
         else Over_limit(1);                        // carrent overlimit
    }


 } 
//******************** END OF INTERRUPS ***************************************

 
 
 
 
 
 
 
 
 //******************** INITIALIZATION from EEprom *****************************

 unsigned char Init_EE(struct Sdata *s)
 {
 const unsigned char EConstant[] = {1,128,128,220,230,60,30}; // defaults
                             /* 1. Regime  -  1,  2 (voltage, power)
                                2. CalibrU - 91 - 182;
                                3. CalibrI - 91 - 182;
                                4. Out_U   - 50 - 250;
                                5. FStartU - 150- 250; ( forse start Voltage)
                                6. FStartTime- 0- 255; ( forse start time)
                                7. TimeOff -  5 - 95;  ( time to off x/10 h), 0-disable)
                                */

 unsigned char i, x, EeWill;
 EeWill = Eeprom_Read_(0);  // EEprom data available?
 if (EeWill != 10)       // if not - write defaults
    {
     Eeprom_Write_(0,10);
     for (i=0; i<7; i++) {
      x = EConstant[i];
      Eeprom_Write_(i+1, x);
     Pause_ms(20);
     }
    }


 s->CalibrU = Eeprom_Read_(2);
 s->CalibrI = Eeprom_Read_(3);
 s->Out_U = Eeprom_Read_(4);
 s->FStartU = Eeprom_Read_(5);
 s->FStartTime = Eeprom_Read_(6);
 s->TimeOff = Eeprom_Read_(7);
 Regime = Eeprom_Read_(1);     // Read saved data

 while (1) {
  if ((s->CalibrU < 91)||(s->CalibrU > 181)) break;
  if ((s->CalibrI < 91)||(s->CalibrI > 181)) break;
  if ((s->Out_U < 50)||(s->Out_U > 250)) break;
  if ((s->FStartU < 150)||(s->FStartU > 250)) break;
  if ((Regime > 2)||(Regime==0)) break;

  return 1; // all is Ok. Exit
  }
  
 Regime = EConstant[0];            //
 s->CalibrU = EConstant[1];        //
 s->CalibrI = EConstant[2];        //
 s->Out_U = EConstant[3];          //
 s->FStartU = EConstant[4];        //
 s->FStartTime = EConstant[5];     //
 s->TimeOff = EConstant[6];        // replace by default

 return 0; // Exit with EEprom problems
 }
 //*****************************************************************************
 





//***************** Menu of change the Regime and set parameters ***************
// in - point of stucture Sdata
// push the both buttons - next menu

 void Menu_regime (struct Sdata *s) {

 Flags &= 0b11100000;       // clear button flags
 TimeCount = 0;

 while (((short)TimeCount).F2==0) {
    if (!s1_on && !s2_on) {
      TimeCount = 0;

      Sign[3] = _U;       //   'U__P' to ind.
      Sign[2] = _Bottom;  //
      Sign[1] = _Bottom;  //
      Sign[0] = _P;       //

      while (((short)TimeCount).F3==0) {
         if (Regime.F0==1) FlashFlags = 8;  //
            else FlashFlags = 1;          // The current regime is flash

         if ((Flags.F0==1) || s1_on || s2_on ) Regime=Button_push(Regime,1,2);
         if (Flags.F4==1) break;
        }                      // exit after 8s or by push the both buttons
      Not_Pushed() ;           // not pushed waiting
      FlashFlags = 0;
      Flags &= 0b11100000;       // clear button flags
      return ; //
      }
     }
 
 PWM1_10_off();             // off hiter current

 Menu(s);     // to parameters Menu (s - pointer of struct Sdata)

 Not_Pushed() ;    // not pushed waiting
 FlashFlags=0;
 Flags &= 0b11100000;       // clear button flags

 PWM1_10_on();             // on hiter current
 }


 //****************************************************************************
 
 
 
 
 
//************************************* Menu ***********************************
// Changing parameters FStartU, FStartTime, TimeOff
// Starting the calibration menu


void Menu (struct Sdata *s) {
  unsigned char MenuItem=1, Llimit, Hlimit, Temp, *S_value;

 Flags &= 0b11100000;       // clear button flags
 PWM1_10_off();             // off hiter current

 Sign[3]=_F;
 Sign[2]=_S;
 Sign[1]=_t;
 Sign[0]=_U;
 point(1);       //  FSt.U
 
 Not_Pushed() ;    // not pushed buttons waiting
 TimeCount = 0;

 while (((short)TimeCount).F3==0) {   // 8s loop

 switch  (MenuItem) {
  case 1:  {
     Sign[3]=_F;
     Sign[2]=_S;
     Sign[1]=_t;
     Sign[0]=_U;
     point(1);       //  FSt.U
     break;
     }
  case 2:  {
     Sign[3]=_F;
     Sign[2]=_S;
     Sign[1]=_t;
     Sign[0]=_t;
     point(1);       //  FSt.t
     break;
     }
   case 3:  {
     Sign[3]=_t;
     Sign[2]=_o;
     Sign[1]=_F;
     Sign[0]=_F;
     point(3);       //  t.oFF
     break;
     }
   case 4:  {
     Sign[3]=_C;
     Sign[2]=_A;
     Sign[1]=_L;
     Sign[0]=_I;          // CALI
     break;
     }
   }

  if ((Flags.F0==1) || s1_on || s2_on ) MenuItem=Button_push(MenuItem,1,4);

  if (Flags.F4==1) {
         // here is begining of parameters changing ----------------------------

      FlashFlags = 8;   // flash 1-th col. ind.
      Flags &= 0b11100000;       // clear button Flags

      switch (MenuItem) {
         case 1: {
             Sign[3] =_u; S_value = &(s->FStartU); Llimit = 50; Hlimit = 250;
             point(3); break;                 // Fstart voltage, 50-250 V
             }
         case 2: {
             Sign[3] =_t; S_value = &(s->FStartTime); Llimit =0; Hlimit = 240;
             point(3);break;                     // Fstart time, 0-240 s
             }
         case 3: {
             Sign[3] =_t; S_value = &(s->TimeOff); Llimit = 5; Hlimit = 95;
             set(FlashFlags,2);                   // time to off, 0.5-9.5 hours
             }
         }

        if (MenuItem==4) {                              // to the calibration
           Calibration(&(s->CalibrU), &(s->CalibrI));  //  and exit
           break;                                     //
          }
        Temp = (*S_value);
        Int_to_Ind(Temp);
        if (MenuItem==3) {Sign[2]=_o; point(1);}     // comma to 1-st
        Not_Pushed() ;   // not pushed buttons waiting

        TimeCount = 0;        //
        Flags &= 0b11100000;  // clear time and flags
        
        while (1) {   // 8s loop

           if (Flags.F4==1) break;

           Int_to_Ind(Temp);                                // value to und.
           if (MenuItem==3) {Sign[2]=_o; point(1);}   // comma to 1-st

           if ((Flags.F0==1)||s1_on||s2_on ) Temp=Button_push(Temp,Llimit,Hlimit);

           if ((Flags.F0==0)&&(((short)TimeCount).F3==1))  break;
           
          }
          
         Not_Pushed() ;   // not pushed buttons waiting

         if (Temp!=(*S_value)) {
            (*S_value) = Temp;               //
            Eeprom_Write_(MenuItem+4,Temp);  // save changed value
           }
     
         TimeCount = 0;        //
         FlashFlags = 0;       //
         Flags &= 0b11100000;  // clear time and flags

    }
 //------------------------------------------------------------------------------


 }




}
//******************************************************************************